use strictures;
use Devel::Dwarn;

use Data::Query::ExprDeclare qw(SELECT FROM WHERE JOIN ON ORDER BY DESC NULLS LAST AS LEFT JOIN OFFSET LIMIT);

BEGIN { require 't/example.to_sql' };

make_renderer;

DwarnL to_sql(SELECT { $_->foo, "bar" });
DwarnL to_sql(SELECT { $_->foo, "bar" } FROM { $_->baz });
DwarnL to_sql(SELECT { $_->foo } FROM { $_->baz, AS('quux') });
DwarnL to_sql(
  SELECT { $_->cd->name } FROM { $_->cds, AS('cd') }
);

my $w_join =
  SELECT { $_->cd->name }
  FROM { $_->cds, AS 'cd' }
  JOIN { $_->artists, AS 'artist' }
    ON { $_->cd->artistid eq $_->artist->id };

DwarnL to_sql($w_join);

Dwarn to_sql(FROM { $w_join } WHERE { $_->cd->year > 2000 });

DwarnL to_sql(
  SELECT { $_->artist->name }
  FROM { $_->artists, AS 'artist' }
  LEFT JOIN { $_->cds, AS 'cd' }
         ON { $_->cd->artistid eq $_->artist->id }
);
DwarnL to_sql(
  SELECT { $_->artist->name } WHERE { $_->artist->age > 25 }
);
DwarnL to_sql(
  SELECT { $_->cd->name }
  FROM { $_->cds, AS 'cd' }
  JOIN { $_->artists, AS 'artist' }
    ON { $_->cd->artistid eq $_->artist->id }
  WHERE { $_->artist->age > 25 }
);
DwarnL to_sql(ORDER BY { $_->foo, $_->bar, $_->baz } DESC);

my $basic = SELECT { $_->cd->name } FROM { $_->cds, AS 'cd' };

DwarnL to_sql(
  FROM { $basic } WHERE { $_->year > 2000 }
);

DwarnL to_sql(
  FROM { $basic } ORDER BY { $_->year }
);

DwarnL to_sql(
  SELECT { $_->cd->name } FROM { $_->cds, AS 'cd' } ORDER BY { $_->year } DESC NULLS LAST
);

my $lo = 
  SELECT { $_->cd->name }
  FROM { $_->cds, AS 'cd' }
  ORDER BY { $_->year }
  LIMIT { 10 } OFFSET { 10 };

sub render_orders {

  make_renderer('LimitOffset');

  DwarnL to_sql($lo);

  make_renderer('RowNum');

  DwarnL to_sql($lo);

  make_renderer('RowNumberOver');

  DwarnL to_sql($lo);

}

render_orders();

$lo =
  SELECT { $_->cd->name, $_->artist->name }
  FROM { $_->cds, AS 'cd' }
  JOIN { $_->artists, AS 'artist' }
    ON { $_->cd->artistid eq $_->artist->id }
  WHERE { $_->artist->age > 25 }
  ORDER BY { $_->artist->age, $_->cd->id }
  LIMIT { 10 } OFFSET { 20 };

render_orders();

my @cols = qw(name year genre);

DwarnL to_sql(
  SELECT { my $cd = $_->cd; map $cd->$_, @cols }
  FROM { $_->cds, AS 'cd' }
);
